
# q-expansion of newform 225.4.b.h, downloaded from the LMFDB on 02 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 225
weight = 4
poly_data = [25, 0, -9, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, -4, 0, 1], 5], [[0, 14, 0, -1], 5], [[-9, 0, 2, 0], 1]]

hecke_ring_character_values = [[101, [1, 0, 0, 0]], [127, [-1, 0, 0, 0]]]
aps_data = [[0, -1, 0, 1], [0, 0, 0, 0], [0, 0, 0, 0], [0, -13, 0, -4], [-14, 0, -4, 0], [0, 9, 0, -16], [0, -34, 0, -20], [-3, 0, 4, 0], [0, -66, 0, 12], [46, 0, -28, 0], [61, 0, 28, 0], [0, 142, 0, -24], [-196, 0, 52, 0], [0, 345, 0, -4], [0, -310, 0, -32], [0, 424, 0, -28], [62, 0, 64, 0], [375, 0, 56, 0], [0, 179, 0, -100], [-412, 0, -20, 0], [0, -54, 0, 8], [440, 0, -160, 0], [0, -78, 0, -192], [-432, 0, -144, 0], [0, -521, 0, 0], [-504, 0, 36, 0], [0, 380, 0, -264], [0, -840, 0, -132], [-1013, 0, 24, 0], [0, 1492, 0, 128], [0, -456, 0, 40], [960, 0, -276, 0], [0, 1014, 0, -24], [1108, 0, 216, 0], [1474, 0, 488, 0], [-273, 0, 380, 0], [0, -1019, 0, 264], [0, 1067, 0, 420], [0, -1152, 0, 324], [0, 942, 0, 228], [-1690, 0, 520, 0], [-79, 0, -432, 0], [326, 0, 916, 0], [0, -1357, 0, 312], [0, 1302, 0, -168], [2301, 0, 172, 0], [1571, 0, -372, 0], [0, 1949, 0, 804], [0, -5492, 0, 164], [-1297, 0, 776, 0], [0, -1620, 0, -1116], [-712, 0, 676, 0], [-231, 0, -496, 0], [3228, 0, -600, 0], [0, 2100, 0, -552], [0, -2812, 0, 580], [2618, 0, -44, 0], [1464, 0, -376, 0], [0, 6203, 0, 368], [-1578, 0, 228, 0], [0, -161, 0, -1316], [0, -3702, 0, 480], [0, 585, 0, -188], [2946, 0, -264, 0], [0, -3733, 0, -1080], [0, 4240, 0, 1448], [292, 0, 1480, 0], [0, 2823, 0, 1288], [0, -454, 0, -320], [2334, 0, -272, 0], [0, 366, 0, 1236], [6132, 0, -156, 0], [0, -6067, 0, -532], [0, 915, 0, 416], [3015, 0, -1460, 0], [0, -492, 0, -2280], [1092, 0, 2004, 0], [0, 6225, 0, -1328], [11520, 0, 684, 0], [12163, 0, -24, 0], [10768, 0, -1084, 0], [-1242, 0, -2008, 0], [3174, 0, -2868, 0], [0, -8733, 0, 200], [-525, 0, -1580, 0], [0, 1056, 0, 456], [-14116, 0, 1048, 0], [0, -918, 0, -2864], [-7632, 0, 2700, 0], [0, -6200, 0, -1784], [0, -3770, 0, -1612], [11430, 0, 720, 0], [0, 3349, 0, 1860], [7880, 0, 2704, 0], [-2643, 0, -1316, 0], [0, 16916, 0, -284], [-7378, 0, -2216, 0], [12734, 0, -908, 0], [0, -2985, 0, 3236], [13501, 0, 1888, 0], [0, -4128, 0, 2536], [0, 2826, 0, -2160], [0, -9018, 0, 1188], [-5862, 0, -3444, 0], [22379, 0, -276, 0], [0, 5509, 0, 2160], [0, -360, 0, 5868], [0, -3460, 0, -356], [724, 0, 968, 0], [-21301, 0, -696, 0], [0, -20256, 0, -2000], [0, 18138, 0, -688], [0, 14784, 0, -204], [-9167, 0, 3156, 0], [6057, 0, -700, 0], [-7236, 0, 2952, 0], [0, 8620, 0, 616], [0, -11404, 0, 640], [0, -3922, 0, -2000], [9020, 0, 1060, 0], [3522, 0, -4720, 0], [0, -22034, 0, 1288], [0, -18138, 0, 2172], [0, 4488, 0, -2400], [-3216, 0, -2776, 0], [-22570, 0, -2756, 0], [-18273, 0, 1624, 0], [-2886, 0, -2892, 0], [0, 16891, 0, -2436], [0, 8870, 0, 3696], [23268, 0, 1696, 0], [0, 21460, 0, -976], [4344, 0, 3224, 0], [0, -6369, 0, -3656], [-1068, 0, -3072, 0], [18661, 0, 4872, 0], [0, -6732, 0, -3060], [0, -13575, 0, 652], [0, -9140, 0, -1192], [1480, 0, 4460, 0], [-173, 0, 2980, 0], [-16878, 0, 2208, 0], [0, 21113, 0, 3252], [0, 4496, 0, -8420], [-14650, 0, -3520, 0], [-21050, 0, -3340, 0], [0, -6173, 0, -1160], [0, 33260, 0, -2552], [13200, 0, -2840, 0], [0, 5720, 0, -1256], [0, -16003, 0, -4984], [6472, 0, -6472, 0], [0, -3175, 0, -9444], [0, -14490, 0, 3168], [0, 6020, 0, -2608], [-19022, 0, 1040, 0], [14265, 0, 1900, 0], [3298, 0, -3964, 0], [0, -7567, 0, -4312], [-11926, 0, 6112, 0], [0, -3606, 0, -1884], [0, 34696, 0, 2096], [0, -24296, 0, 3120], [9414, 0, -8568, 0], [0, 13706, 0, 2440], [0, -14436, 0, 432], [-20263, 0, -5820, 0], [0, 13646, 0, 4784]]
